searchData={"items":[{"type":"module","title":"RCrypto","doc":"This module contains the NIF function to the rust library.","ref":"RCrypto.html"},{"type":"function","title":"RCrypto.decrypt_aes_key/2","doc":"Decrypts the encrypted AES key encoded as Base64 string using the private reference.","ref":"RCrypto.html#decrypt_aes_key/2"},{"type":"function","title":"RCrypto.fetch_private_key/2","doc":"This function fetches the private key from the pem string. The password is optional.","ref":"RCrypto.html#fetch_private_key/2"},{"type":"module","title":"WhatsappFlowCrypto","doc":"Load the private key from the file system. The private key is reused once loaded. The library returns a reference\n  to the private key. This can be done by the function `fetch_private_key/2`. Just read the private key and\n  call the function to receive a reference of the private key. If the private key is encrypted you need to\n  specify the password.\n\nFor each request, we do the encryption like this:\n  1. Decrypt the encrypted AES key\n  2. Decrypt the request with initial vector and the encrypted AES key\n  3. Encrypt the response with the inverted initial vector and the encrypted AES key\n\n\n    {:ok, private_key_pem} = File.read(private_pem_path);\n    WhatsappFlowCrypto.fetch_private_key(private_key_pem, \"test\")\n    {:ok, #Reference<0.1652491651.443678740.227250>}\n\nAfter this, you can decrypt the request. The request contains\n  * the encrypted AES key\n  * the initial vector\n  * the encrypted flow data\n\nAll values are base64 encoded.\n    WhatsappFlowCrypto.decrypt_request(private_key_ref, encrypted_aes_key, initial_vector, encrypted_flow_data)\n    {:ok, {decrypted_body, aes_key, initial_vector}}\n\n    WhatsappFlowCrypto.encrypt_response(aes_key, initial_vector, response)\n    \"KUkRnUDAUKqhiovnQ9RRwmdBjcg87/wh+ZrMtbh8xlx3\"","ref":"WhatsappFlowCrypto.html"},{"type":"function","title":"WhatsappFlowCrypto.decrypt_request/4","doc":"Decrypts the encrypted body and AES key using the private key reference. It returns the decrypted JSON map, the decrypted AES key and the initial vector.\nThey are need to decrypt the response.","ref":"WhatsappFlowCrypto.html#decrypt_request/4"},{"type":"function","title":"WhatsappFlowCrypto.encrypt_response/3","doc":"Encrypts the response using the decrypted_aes_key and initial vector of the request. The response will be encoded as JSON.","ref":"WhatsappFlowCrypto.html#encrypt_response/3"},{"type":"function","title":"WhatsappFlowCrypto.fetch_private_key/2","doc":"Fetches the private key from the pem. The password is optional and only needed if the\n  private key is encrypted. Heads up: some cipher are not supported by this library. The DES3 cipher\n  is unsupported. You can use another cipher (AES-128) to encrypt the private key.","ref":"WhatsappFlowCrypto.html#fetch_private_key/2"},{"type":"extras","title":"WhatsappFlowCrypto","doc":"# WhatsappFlowCrypto\n\nIf you want to implement a [WhatsApp Flow-Endpoint](https://developers.facebook.com/docs/whatsapp/flows/guides/implementingyourflowendpoint)\nyou need to implement the specified workflow to [decrypt the request and encrypt the response.](https://developers.facebook.com/docs/whatsapp/flows/guides/implementingyourflowendpoint#request-decryption-and-encryption)\n\nFor many tasks you can use the Erlang [`crypto`](https://www.erlang.org/doc/apps/crypto/crypto.html) module, but unfortunately this is not possible at the time of writing \nin this case because the necessary configuration of the block cipher is not supported:\n\nTo decrypt the AES key you need \n\n> RSA/ECB/OAEPWithSHA-256AndMGF1Padding algorithm with SHA256 as a hash function for MGF1;\n\nOnce the AES key is decrypted we can do the rest of the workflow using the `crypto` module.\n\nThis package contains a Rust implementation to decrypt the encrypted AES key. With the help \nof [Rustler](https://github.com/rusterlium/rustler) a safe NIF binding is created\nand the missing function can be compensated quickly and efficiently.","ref":"readme.html"},{"type":"extras","title":"Installation - WhatsappFlowCrypto","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed\nby adding `whatsapp_flow_crypto` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:whatsapp_flow_crypto, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Private key - WhatsappFlowCrypto","doc":"The package provides a function to extract the private key (with or without a password) from a pem string.\n\nTo create the private/public key you can use then `openssl` tool. In the following example\nwe will create an encrypted private key using the AES-128 cipher (**DES3 cipher is not supported anymore**).\nThe last command exports the private key without encryption.\n\n```bash\nopenssl genrsa -aes128 -out private.pem 2048\nopenssl rsa -in private.pem -outform PEM -pubout -out public.pem\nopenssl pkcs8 -topk8 -inform PEM -outform PEM -nocrypt -in private.pem -out unencrypted-private.pem\n```","ref":"readme.html#private-key"},{"type":"extras","title":"Basic usage - WhatsappFlowCrypto","doc":"Loading the private key:\n```elixir\n{:ok, private_key_pem} = File.read(private_pem_path);\nWhatsappFlowCrypto.fetch_private_key(private_key_pem, \"test\")\n{:ok, #Reference<0.1652491651.443678740.227250>}\n\n```\nDecrypt the request and encrypt the response:\n```elixir\nWhatsappFlowCrypto.decrypt_request(private_key_ref, encrypted_aes_key, initial_vector, encrypted_flow_data)\n{:ok, {decrypted_body, aes_key, initial_vector}}\n\nWhatsappFlowCrypto.encrypt_response(aes_key, initial_vector, %{\"Hello\" => \"World\"})\n\"KUkRnUDAUKqhiovnQ9RRwmdBjcg87/wh+ZrMtbh8xlx3\"\n```\n\nFor the detail see the `WhatsappFlowCrypto`.\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc)\nand published on [HexDocs](https://hexdocs.pm). Once published, the docs can\nbe found at  .","ref":"readme.html#basic-usage"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.1.0 (2024-10-23) - Changelog","doc":"Initial release","ref":"changelog.html#0-1-0-2024-10-23"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,50]}}